"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var iq_datepicker_options_1 = require("../iq-datepicker/iq-datepicker-options");
var IqCalendarComponent = (function () {
    function IqCalendarComponent() {
        this.dateSelected = new core_1.EventEmitter();
    }
    IqCalendarComponent.prototype.ngOnInit = function () {
        this.date = this.selectedDate ? new Date(this.selectedDate.getTime()) : new Date();
        this.updateViewDays();
    };
    IqCalendarComponent.prototype.prevMonth = function () {
        var currentMonth = this.date.getMonth();
        if (currentMonth === 0) {
            this.date.setMonth(11);
            this.prevYear();
        }
        else {
            this.date.setMonth(currentMonth - 1);
        }
        this.updateViewDays();
    };
    IqCalendarComponent.prototype.nextMonth = function () {
        var currentMonth = this.date.getMonth();
        if (currentMonth === 11) {
            this.date.setMonth(0);
            this.nextYear();
        }
        else {
            this.date.setMonth(currentMonth + 1);
        }
        this.updateViewDays();
    };
    IqCalendarComponent.prototype.prevYear = function () {
        this.date.setFullYear(this.date.getFullYear() - 1);
        this.updateViewDays();
    };
    IqCalendarComponent.prototype.nextYear = function () {
        this.date.setFullYear(this.date.getFullYear() + 1);
        this.updateViewDays();
    };
    IqCalendarComponent.prototype.updateViewDays = function () {
        this.weeks = [];
        var firstDayDate = new Date(this.date.getFullYear(), this.date.getMonth(), 1);
        var offset = firstDayDate.getDay();
        this.populatePreviousMonthDates(offset);
        var dayNumber = this.populateCurrentMonthDates(offset);
        this.populateNextMonthDates(dayNumber);
    };
    IqCalendarComponent.prototype.populateNextMonthDates = function (dayNumber) {
        if (dayNumber < 6) {
            for (var i = ++dayNumber; i <= 6; i++) {
                this.weeks[this.weeks.length - 1][i] = {
                    currentMonth: false,
                    currentDate: false,
                    prevMonth: false,
                    nextMonth: true,
                    isSelectedDate: this.isSelectedDate(this.getNextMonth(), i - dayNumber + 1),
                    value: i - dayNumber + 1
                };
            }
        }
    };
    IqCalendarComponent.prototype.getPreviousMonth = function () {
        return this.date.getMonth() === 0 ? 11 : this.date.getMonth() - 1;
    };
    IqCalendarComponent.prototype.getNextMonth = function () {
        return this.date.getMonth() === 11 ? 0 : this.date.getMonth() + 1;
    };
    IqCalendarComponent.prototype.populateCurrentMonthDates = function (offset) {
        var lastDayDate = new Date(this.date.getFullYear(), this.date.getMonth() + 1, 0);
        var numberOfDaysInMonth = lastDayDate.getDate();
        var dayNumber;
        for (var i = 0; i < numberOfDaysInMonth; i++) {
            var week = Math.floor((i + offset) / 7);
            dayNumber = (i + offset) % 7;
            if (!this.weeks[week]) {
                this.weeks.push([]);
            }
            this.weeks[week][dayNumber] = {
                currentMonth: true,
                currentDate: this.isCurrentDate(i + 1),
                prevMonth: false,
                nextMonth: false,
                isSelectedDate: this.isSelectedDate(this.date.getMonth(), i + 1),
                value: i + 1
            };
        }
        return dayNumber;
    };
    IqCalendarComponent.prototype.isCurrentDate = function (dayNumber) {
        var today = new Date();
        return this.date.getFullYear() === today.getFullYear()
            && this.date.getMonth() === today.getMonth()
            && dayNumber === today.getDate();
    };
    IqCalendarComponent.prototype.populatePreviousMonthDates = function (offset) {
        if (offset > 0) {
            this.weeks.push([]);
            for (var i = 0; i <= offset; i++) {
                var lastDayPreviousMonth = new Date(this.date.getFullYear(), this.date.getMonth(), 0);
                this.weeks[0][offset - i] = {
                    currentMonth: false,
                    currentDate: false,
                    prevMonth: true,
                    nextMonth: false,
                    isSelectedDate: this.isSelectedDate(this.getPreviousMonth(), lastDayPreviousMonth.getDate() - i + 1),
                    value: lastDayPreviousMonth.getDate() - i + 1
                };
            }
        }
    };
    IqCalendarComponent.prototype.selectMonth = function (index) {
        this.date.setMonth(index);
        this.updateViewDays();
    };
    IqCalendarComponent.prototype.setYear = function (year) {
        if (!isNaN(year) && year >= 1970 && year <= 9999) {
            this.date.setFullYear(year);
            this.updateViewDays();
        }
        else {
            this.date.setFullYear(new Date().getFullYear());
        }
    };
    IqCalendarComponent.prototype.onDateSelected = function (date, prevMonth, nextMonth) {
        this.date.setDate(date);
        if (prevMonth) {
            this.prevMonth();
        }
        if (nextMonth) {
            this.nextMonth();
        }
        this.selectedDate = this.date;
        this.dateSelected.emit(this.date);
        this.updateViewDays();
    };
    IqCalendarComponent.prototype.isSelectedDate = function (month, date) {
        if (!this.selectedDate) {
            return false;
        }
        else {
            return this.selectedDate.getFullYear() === this.date.getFullYear()
                && this.selectedDate.getMonth() === month
                && this.selectedDate.getDate() === date;
        }
    };
    IqCalendarComponent.prototype.onKeyDown = function (event) {
        if (event.keyCode === 13) {
            this.setYear(event.target.value);
            event.stopPropagation();
        }
    };
    return IqCalendarComponent;
}());
__decorate([
    core_1.Input(),
    __metadata("design:type", iq_datepicker_options_1.IqDatepickerOptions)
], IqCalendarComponent.prototype, "options", void 0);
__decorate([
    core_1.Input(),
    __metadata("design:type", Object)
], IqCalendarComponent.prototype, "translations", void 0);
__decorate([
    core_1.Input(),
    __metadata("design:type", Date)
], IqCalendarComponent.prototype, "date", void 0);
__decorate([
    core_1.Input(),
    __metadata("design:type", Date)
], IqCalendarComponent.prototype, "selectedDate", void 0);
__decorate([
    core_1.Output(),
    __metadata("design:type", Object)
], IqCalendarComponent.prototype, "dateSelected", void 0);
IqCalendarComponent = __decorate([
    core_1.Component({
        selector: 'iq-calendar',
        templateUrl: './iq-calendar.component.html',
        styleUrls: ['./iq-calendar.component.css']
    }),
    __metadata("design:paramtypes", [])
], IqCalendarComponent);
exports.IqCalendarComponent = IqCalendarComponent;
//# sourceMappingURL=iq-calendar.component.js.map