"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var iq_datepicker_options_1 = require("./iq-datepicker-options");
var iq_datepicker_english_translation_1 = require("./iq-datepicker-english-translation");
var forms_1 = require("@angular/forms");
var iq_calendar_component_1 = require("../iq-calendar/iq-calendar.component");
require("rxjs/add/operator/debounceTime");
require("rxjs/add/operator/distinctUntilChanged");
var KEY_CODE_ENTER = 13;
var KEY_CODE_TAB = 9;
var VALUE_ACCESSOR = {
    provide: forms_1.NG_VALUE_ACCESSOR,
    useExisting: core_1.forwardRef(function () { return IqDatepickerComponent; }),
    multi: true
};
var IqDatepickerComponent = (function () {
    function IqDatepickerComponent(elementRef) {
        this.elementRef = elementRef;
        this.options = {};
        this.translations = new iq_datepicker_english_translation_1.IqDatepickerEnglishTranslation();
        this.dateSelected = new core_1.EventEmitter();
        this.selectedDateInput = new forms_1.FormControl();
        this.calendarVisible = false;
        this.disabled = false;
        this.dateMask = [];
        this.yearMask = [/[1-3]/, /\d/, /\d/, /\d/];
        this.monthMask = [/[0-1]/, /\d/];
        this.dayMask = [/[0-3]/, /\d/];
        this.propagateChange = function (_) {
        };
        this.defaults = {
            size: 'md',
            calendarBtnClass: 'btn btn-default',
            calendarBtnIcon: 'glyphicon glyphicon-calendar',
            removeBtnClass: 'btn btn-default',
            removeBtnVisible: true,
            removeBtnIcon: 'glyphicon glyphicon-remove',
            horizontal: false,
            dateFormat: 'dd/mm/yyyy',
            showPlaceholder: true
        };
    }
    IqDatepickerComponent.prototype.writeValue = function (obj) {
        var date = obj;
        if (typeof obj === 'string' || typeof obj === 'number') {
            date = new Date(date);
        }
        this.onDateSelected(date);
    };
    IqDatepickerComponent.prototype.registerOnChange = function (fn) {
        this.propagateChange = fn;
    };
    IqDatepickerComponent.prototype.registerOnTouched = function (fn) {
    };
    IqDatepickerComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
        this.selectedDateInput.disable(isDisabled);
    };
    IqDatepickerComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.setDefaults();
        this.buildDateMask();
        this.selectedDateInput
            .valueChanges
            .debounceTime(100)
            .distinctUntilChanged()
            .subscribe(function (newValue) {
            if (newValue) {
                var year = newValue.substring(_this.yearIndex, _this.yearIndex + 4);
                var month = newValue.substring(_this.monthIndex, _this.monthIndex + 2);
                var day = newValue.substring(_this.dayIndex, _this.dayIndex + 2);
                if (!isNaN(year) && _this.calendarComponent) {
                    _this.calendarComponent.setYear(year);
                }
                if (!isNaN(month) && _this.calendarComponent) {
                    _this.calendarComponent.selectMonth(month - 1);
                }
                if (!isNaN(year) && !isNaN(month) && !isNaN(day)) {
                    var date = new Date(year, month - 1, day);
                    _this.onDateSelected(date);
                }
            }
        });
        this.yearIndex = this.options.dateFormat.indexOf('yyyy');
        this.monthIndex = this.options.dateFormat.indexOf('mm');
        this.dayIndex = this.options.dateFormat.indexOf('dd');
    };
    IqDatepickerComponent.prototype.setDefaults = function () {
        var _this = this;
        Object
            .keys(this.defaults)
            .filter(function (key) { return _this.options[key] === undefined || _this.options[key] === null; })
            .forEach(function (key) { return _this.options[key] = _this.defaults[key]; });
    };
    IqDatepickerComponent.prototype.onDateSelected = function (date) {
        this.selectedDateInput.patchValue(this.dateToString(date));
        this.selectedDate = date ? new Date(date.getTime()) : null;
        this.propagateChange(this.selectedDate);
        this.dateSelected.emit(this.selectedDate);
        this.hideCalendar();
    };
    IqDatepickerComponent.prototype.dateToString = function (date) {
        if (date) {
            var dateStr = this.options.dateFormat;
            var month = date.getMonth() + 1;
            var day = date.getDate();
            dateStr = dateStr.replace(/dd/g, day < 10 ? '0' + day : String(day));
            dateStr = dateStr.replace(/mm/g, month < 10 ? '0' + month : String(month));
            dateStr = dateStr.replace(/yyyy/g, String(date.getFullYear()));
            return dateStr;
        }
        else {
            return null;
        }
    };
    IqDatepickerComponent.prototype.clear = function () {
        this.onDateSelected(null);
        if (this.calendarComponent) {
            this.calendarComponent.selectedDate = null;
            this.calendarComponent.updateViewDays();
        }
    };
    IqDatepickerComponent.prototype.showCalendar = function () {
        this.calendarVisible = true;
    };
    IqDatepickerComponent.prototype.hideCalendar = function () {
        this.calendarVisible = false;
    };
    IqDatepickerComponent.prototype.toggleCalendar = function () {
        this.calendarVisible = !this.calendarVisible;
    };
    IqDatepickerComponent.prototype.handleKeyDown = function (event) {
        if (event.keyCode === KEY_CODE_ENTER || event.keyCode === KEY_CODE_TAB) {
            this.hideCalendar();
        }
    };
    IqDatepickerComponent.prototype.handleClick = function (event) {
        if (this.calendarVisible && !this.elementRef.nativeElement.contains(event.target)) {
            this.hideCalendar();
        }
    };
    IqDatepickerComponent.prototype.buildDateMask = function () {
        var _this = this;
        var items = [];
        items[this.options.dateFormat.indexOf('dd')] = this.dayMask;
        items[this.options.dateFormat.indexOf('mm')] = this.monthMask;
        items[this.options.dateFormat.indexOf('yyyy')] = this.yearMask;
        Object.keys(items).forEach(function (key, i) {
            if (i > 0) {
                _this.dateMask.push(['/']);
            }
            items[key].forEach(function (item) { return _this.dateMask.push(item); });
        });
    };
    return IqDatepickerComponent;
}());
__decorate([
    core_1.Input(),
    __metadata("design:type", iq_datepicker_options_1.IqDatepickerOptions)
], IqDatepickerComponent.prototype, "options", void 0);
__decorate([
    core_1.Input(),
    __metadata("design:type", Object)
], IqDatepickerComponent.prototype, "translations", void 0);
__decorate([
    core_1.Output(),
    __metadata("design:type", Object)
], IqDatepickerComponent.prototype, "dateSelected", void 0);
__decorate([
    core_1.ViewChild(iq_calendar_component_1.IqCalendarComponent),
    __metadata("design:type", Object)
], IqDatepickerComponent.prototype, "calendarComponent", void 0);
__decorate([
    core_1.HostListener('keydown', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], IqDatepickerComponent.prototype, "handleKeyDown", null);
__decorate([
    core_1.HostListener('document:click', ['$event']),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", void 0)
], IqDatepickerComponent.prototype, "handleClick", null);
IqDatepickerComponent = __decorate([
    core_1.Component({
        selector: 'iq-datepicker',
        templateUrl: './iq-datepicker.component.html',
        styleUrls: ['./iq-datepicker.component.css'],
        providers: [VALUE_ACCESSOR]
    }),
    __metadata("design:paramtypes", [core_1.ElementRef])
], IqDatepickerComponent);
exports.IqDatepickerComponent = IqDatepickerComponent;
//# sourceMappingURL=iq-datepicker.component.js.map